<?php

namespace Modules\Solicitacao\Modules\Relatorios\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Request\Filter;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\ParameterBag;

class Sup97000VerificacaoCadastrosProcessosComprasSuprimentos extends \Teknisa\GER\Controllers\TeknisaCrud
{

    protected $environment;
    protected $connection;
    protected $dataSourceManager;
    protected $dataSourceName = "sup08200_relatorio";
    protected $unitData;

    public function __construct($crudService, $parameterBag, $dataSourceManager, $environment, $connection)
    {
        $this->crudService       = $crudService;
        $this->parameterBag =   $parameterBag;
        $this->dataSourceManager = $dataSourceManager;
        $this->environment       = $environment;
        $this->connection        = $connection;
        parent::__construct($crudService, $parameterBag);
    }

    private function getData($filterCriteria, $request)
    {
        try {
            $newRequest = new Filter($filterCriteria, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
            $newResponse = new DTO\Response();

            parent::find($newRequest, $newResponse);
            $data = $newResponse->getDataSets();


            if (is_array($data) && count($data)) {
                $data = end($data);
                if ($data->getDataSourceName() === $this->dataSourceName) {
                    return $data->getRows();
                }
            }

            return array();
        } catch (Exception $e) {
            throw $e;
        }
    }

    private function getUnit($row, $request)
    {
        try {
            $this->dataSourceName = 'filial';
            $filterCriteria = new FilterCriteria($this->dataSourceName);
            $filterCriteria->addCondition('CDFILIAL', $row['CDFILIAL']);
            return $this->getData($filterCriteria, $request);
        } catch (Exception $e) {
            throw $e;
        }
    }

    private function isFILIALWithoutCDREGIAO($unitData, &$result)
    {
        try {
            if (!count($result) && is_array($unitData) && count($unitData)) {
                $data = array_filter($unitData, function ($row) {
                    return isset($row['CDREGIAO']);
                }, ARRAY_FILTER_USE_BOTH);

                $isThereData = is_array($data) && count($data);

                if (!$isThereData) {
                    $result = array(
                        "ROUTE"   => "cadSup#cmp02800_regiao",
                        "ERROR"   => "REGION_NOT_REGISTERED_FOR_UNIT",
                        "WINDOW"  => "REGION"
                    );
                }
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    private function isTerminatedUnitContract($unitData, &$result)
    {
        try {
            if (!count($result) && is_array($unitData) && count($unitData)) {
                $data = array_filter($unitData, function ($row) {
                    return $row['IDCNTRRESC'] === 'S';
                }, ARRAY_FILTER_USE_BOTH);

                $isThereData = is_array($data) && count($data);

                if ($isThereData) {
                    $result = array(
                        "ROUTE"   => "cadEst#ger01700_unidade",
                        "ERROR"   => "UNIT_TERMINATED_CONTRACT",
                        "WINDOW"  => "UNIT"
                    );
                }
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    private function unitValidations($row, $request, &$result)
    {
        try {
            $unitData = $this->getUnit($row, $request);
            $this->isFILIALWithoutCDREGIAO($unitData, $result);
            $this->isTerminatedUnitContract($unitData, $result);
            return $unitData;
        } catch (Exception $e) {
            throw $e;
        }
    }

    private function getProdFili($row, $request)
    {
        try {
            $this->dataSourceName = 'produto_prodfili';
            $filterCriteria = new FilterCriteria($this->dataSourceName);
            $filterCriteria->addCondition('CDFILIAL', $row['CDFILIAL']);
            $filterCriteria->addCondition('CDARVPROD', $row['CDARVPROD']);
            return $this->getData($filterCriteria, $request);
        } catch (Exception $e) {
            throw $e;
        }
    }

    private function isProductActive($product, &$result)
    {
        try {
            if (!count($result) && is_array($product) && count($product)) {
                $data = array_filter($product, function ($row) {
                    return $row['IDPRODATIVPR'] === 'S';
                }, ARRAY_FILTER_USE_BOTH);

                $isThereData = is_array($data) && count($data);

                if (!$isThereData) {
                    $result = array(
                        "ROUTE"   => "cadEst_produto#ger01900_cadastroproduto",
                        "ERROR"   => "Produto_Inativo",
                        "WINDOW"  => "PRODUTO"
                    );
                }
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    private function isUnitProductActive($product, &$result)
    {
        try {
            if (!count($result) && is_array($product) && count($product)) {
                $data = array_filter($product, function ($row) {
                    return $row['IDPRODATIV'] === 'A';
                }, ARRAY_FILTER_USE_BOTH);

                $isThereData = is_array($data) && count($data);

                if (!$isThereData) {
                    $result = array(
                        "ROUTE"   => "cadEst#ger03800_produtos_por_unidade",
                        "ERROR"   => "PRODUTO_INATIVO_UNIDADE",
                        "WINDOW"  => "PRODFILI"
                    );
                }
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    private function productValidations($row, $request, &$result)
    {
        try {
            if (!count($result)) {
                $product = $this->getProdFili($row, $request);
                $this->isProductActive($product, $result);
                $this->isUnitProductActive($product, $result);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    private function isCDPERIENTRRegistered($data)
    {
        if (is_array($data) && count($data)) {
            $result = array_filter($data, function ($row) {
                return isset($row['CDPERIENTR']);
            }, ARRAY_FILTER_USE_BOTH);

            return is_array($result) && count($result);
        }
    }

    private function isThereRegisteredData($data)
    {
        if (is_array($data) && count($data)) {
            return true;
        }

        return false;
    }

    private function getPerfilProd($row, $request)
    {
        try {
            $this->dataSourceName = 'produto_perfilprod';
            $filterCriteria = new FilterCriteria($this->dataSourceName);
            $filterCriteria->addCondition('P_CDFILIAL', $row['CDFILIAL']);
            $filterCriteria->addCondition('P_CDARVPROD', $row['CDARVPROD']);
            return $this->getData($filterCriteria, $request);
        } catch (Exception $e) {
            throw $e;
        }
    }

    private function getCalecomp($row, $request)
    {
        try {
            $this->dataSourceName = 'calecomp';
            $filterCriteria = new FilterCriteria($this->dataSourceName);
            $dtEntrega = array(
                $row['INTERVAL_DT']['start'],
                $row['INTERVAL_DT']['end']
            );
            $filterCriteria->addCondition("DTENTREGA", "BETWEEN", $dtEntrega);
            $filterCriteria->addCondition('CDFILIAL', $row['CDFILIAL']);
            $filterCriteria->addCondition('CDPERIENTR', $row['CDPERIENTR']);
            return $this->getData($filterCriteria, $request);
        } catch (Exception $e) {
            throw $e;
        }
    }

    private function perfilProdValidations($row, $request, &$result)
    {
        try {
            if (!count($result)) {
                $perfilProd = $this->getPerfilProd($row, $request);

                if (!count($result) && !$this->isThereRegisteredData($perfilProd)) {
                    $result = array(
                        "ROUTE"   => "sup_cadastros#cmp02500_dados_de_produtos_na_unidade",
                        "ERROR"   => "PROD_NAO_CADASTRADO_UNIDADE",
                        "WINDOW"  => "PERFILPROD"
                    );
                }

                if (!count($result) && !$this->isCDPERIENTRRegistered($perfilProd)) {
                    $result = array(
                        "ROUTE"   => "sup_cadastros#cmp02500_dados_de_produtos_na_unidade",
                        "ERROR"   => "CDPERIENTR_NAO_CADASTRADA_PRODFILI",
                        "WINDOW"  => "PERFILPROD"
                    );
                }

                if (!count($result) && count($perfilProd)) {
                    $row['CDPERIENTR'] = end($perfilProd)['CDPERIENTR'];
                    $calecomp = $this->getCalecomp($row, $request);

                    if (!$this->isThereRegisteredData($calecomp)) {
                        $result = array(
                            "ROUTE"   => "sup_cadastros#cmp01100_calendario_de_compras",
                            "ERROR"   => "CALECOMP_NAO_CALC_PRODUTO",
                            "WINDOW"  => "CALECOMP"
                        );
                    }
                }
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    public function verificaCalculoNecessidadeCompras(DTO\Request $request, DTO\Response $response)
    {
        $row = $request->getRow();
        $result = array();

        try {
            $unitData = $this->unitValidations($row, $request, $result);
            $this->productValidations($row, $request, $result);
            $this->perfilProdValidations($row, $request, $result);

            if (!count($result)) {
                $result = array("ERROR" => "");
            }

            $response->addDataSet(new DataSet('data', $result));
        } catch (Exception $e) {
            $response->setError($e->getMessage());
        }
    }

    public function verificarLancamentoSoliComp(DTO\Request $request, DTO\Response $response)
    {
        $row = $request->getRow();
        $result = array();

        try {
            $unitData = $this->unitValidations($row, $request, $result);
            $this->productValidations($row, $request, $result);

            if (!count($result)) {
                $result = array("ERROR" => "");
            }

            $response->addDataSet(new DataSet('data', $result));
        } catch (Exception $e) {
            $response->setError($e->getMessage());
        }
    }

    private function getParasupri($row, $request)
    {
        try {
            $this->dataSourceName = 'parasupri';
            $filterCriteria = new FilterCriteria($this->dataSourceName);
            $filterCriteria->addCondition('CDFILIAL', $row['CDFILCENTCMP']);
            return $this->getData($filterCriteria, $request);
        } catch (Exception $e) {
            throw $e;
        }
    }

    private function getProdforncomp($row, $request)
    {
        try {
            $this->dataSourceName = 'prodforncomp';
            $filterCriteria = new FilterCriteria($this->dataSourceName);
            $filterCriteria->addCondition('CDPRODCOMPL', $row['CDPRODUTO']);
            $filterCriteria->addCondition('CDFORNECED', $row['CDFORNECED']);
            return $this->getData($filterCriteria, $request);
        } catch (Exception $e) {
            throw $e;
        }
    }

    private function parasupriValidations($row, $request, $unitData, &$result)
    {
        try {
            if (!count($result)) {
                $row['CDFILCENTCMP'] = isset($unitData['CDFILCENTCMP']) ? $unitData['CDFILCENTCMP'] : $unitData['CDFILIAL'];
                $parasupri = $this->getParasupri($row, $request);
                if (!$this->isThereRegisteredData($parasupri)) {
                    $result = array(
                        "ROUTE"   => "sup_parametros#cmp00600_parametros_de_compras_e_suprimentos",
                        "ERROR"   => "NAO_HA_PARAM_CADASTRADOS",
                        "WINDOW"  => "PARASUPRI"
                    );
                }

                return $parasupri;
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    private function getReforeg($row, $request)
    {
        try {
            $this->dataSourceName = 'reforeg';
            $filterCriteria = new FilterCriteria($this->dataSourceName);
            $filterCriteria->addCondition('CDREGIAO', $row['CDREGIAO']);
            $filterCriteria->addCondition('CDFORNECED', $row['CDFORNECED']);
            return $this->getData($filterCriteria, $request);
        } catch (Exception $e) {
            throw $e;
        }
    }

    private function getReforfil($row, $request)
    {
        try {
            $this->dataSourceName = 'reforfil';
            $filterCriteria = new FilterCriteria($this->dataSourceName);
            $filterCriteria->addCondition('CDFILIAL', $row['CDFILIAL']);
            $filterCriteria->addCondition('CDFORNECED', $row['CDFORNECED']);
            $filterCriteria->addCondition('CDPERIENTR', "IS_NULL", "");
            return $this->getData($filterCriteria, $request);
        } catch (Exception $e) {
            throw $e;
        }
    }

    private function fornecedorValidations($row, $request, &$result)
    {
        try {
            if (!count($result)) {
                $reforeg = $this->getReforeg($row, $request);
                if ($this->isThereRegisteredData($reforeg)) {
                    $result = array(
                        "ROUTE"   => "cmp_cadastros#cmp00500_regiao_nao_atendida_pelo_fornecedor",
                        "ERROR"   => "REFOREG_NOT_REGISTERED",
                        "WINDOW"  => "REFOREG"
                    );
                } else {
                    if (!count($result)) {
                        $reforfil = $this->getReforfil($row, $request);
                        if ($this->isThereRegisteredData($reforfil)) {
                            $result = array(
                                "ROUTE"   => "sup_cadastros#cmp00700_filial_nao_atendida_atendida_pelo_fornecedor",
                                "ERROR"   => "NAO_ATENDE_UNIDADE_INFOR",
                                "WINDOW"  => "REFORFIL"
                            );
                        }
                    }
                }
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    private function prodFornValidations($row, $request, &$result)
    {
        try {
            if (!count($result)) {
                $prodforncomp = $this->getProdforncomp($row, $request);
                if (!$this->isThereRegisteredData($prodforncomp)) {
                    $result = array(
                        "ROUTE"   => "cmp_cadastros#cmp00100_produto_por_fornecedor",
                        "ERROR"   => "PRODFORNCOMP_NOT_REGISTERED",
                        "WINDOW"  => "PRODFORNCOMP"
                    );
                }
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    private function getMarcProdComp($row, $request)
    {
        try {
            $this->dataSourceName = 'marcprodcomp';
            $filterCriteria = new FilterCriteria($this->dataSourceName);
            $filterCriteria->addCondition('CDMARCA', $row['CDMARCA']);
            $filterCriteria->addCondition('CDPRMARCOMPL', $row['CDPRODUTO']);
            return $this->getData($filterCriteria, $request);
        } catch (Exception $e) {
            throw $e;
        }
    }

    private function marcprodValidations($row, $request, &$result)
    {
        try {
            if (!count($result)) {
                $marcprodcomp = $this->getMarcProdComp($row, $request);
                if (!$this->isThereRegisteredData($marcprodcomp)) {
                    $result = array(
                        "ROUTE"   => "sup_cadastros#cmp03700_marca",
                        "ERROR"   => "MARCPRODCOMP_NOT_REGISTERED",
                        "WINDOW"  => "MARCPRODCOMP"
                    );
                }
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    private function getProdcomp($row, $request)
    {
        try {
            $this->dataSourceName = 'exists_prodcomp';
            $filterCriteria = new FilterCriteria($this->dataSourceName);
            $filterCriteria->addCondition('CDPRODUTO', $row['CDPRODUTO']);
            $filterCriteria->addCondition('CDOPERADOR', $row['CDOPERADOR']);
            return $this->getData($filterCriteria, $request);
        } catch (Exception $e) {
            throw $e;
        }
    }

    private function prodcompValidations($row, $request, &$result)
    {
        try {
            if (!count($result)) {
                $prodcomp = $this->getProdcomp($row, $request);
                if (!$this->isThereRegisteredData($prodcomp)) {
                    $result = array(
                        "ROUTE"   => "sup_cadastros#cmp01200_comprador",
                        "ERROR"   => "PRODCOMP_NAO_CADASTRADO",
                        "WINDOW"  => "PRODCOMP"
                    );
                }
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    private function getQfomarprod($row, $request)
    {
        try {
            $this->dataSourceName = 'exists_qfomarprod';
            $filterCriteria = new FilterCriteria($this->dataSourceName);
            $filterCriteria->addCondition('CDMARCA', $row['CDMARCA']);
            $filterCriteria->addCondition('CDPRODUTO', $row['CDPRODUTO']);
            $filterCriteria->addCondition('DTINICIAL', $row['INTERVAL_DT']['start']);
            $filterCriteria->addCondition('IDULTIDTVALQ', $row['IDULTIDTVALQ']);
            $filterCriteria->addCondition('IDMARSEMQUA', $row['IDMARSEMQUA']);
            return $this->getData($filterCriteria, $request);
        } catch (Exception $e) {
            throw $e;
        }
    }

    private function getQfoforn($row, $request)
    {
        try {
            $this->dataSourceName = 'exists_qfoforn';
            $filterCriteria = new FilterCriteria($this->dataSourceName);
            $filterCriteria->addCondition('CDFORNECED', $row['CDFORNECED']);
            $filterCriteria->addCondition('DTINICIAL', $row['INTERVAL_DT']['start']);
            $filterCriteria->addCondition('IDULTIDTVALQ', $row['IDULTIDTVALQ']);
            return $this->getData($filterCriteria, $request);
        } catch (Exception $e) {
            throw $e;
        }
    }

    private function getQfoprodforn($row, $request)
    {
        try {
            $this->dataSourceName = 'exists_qfoprodforn';
            $filterCriteria = new FilterCriteria($this->dataSourceName);
            $filterCriteria->addCondition('CDPRODUTO', $row['CDPRODUTO']);
            $filterCriteria->addCondition('CDFORNECED', $row['CDFORNECED']);
            $filterCriteria->addCondition('DTINICIAL', $row['INTERVAL_DT']['start']);
            $filterCriteria->addCondition('IDULTIDTVALQ', $row['IDULTIDTVALQ']);
            return $this->getData($filterCriteria, $request);
        } catch (Exception $e) {
            throw $e;
        }
    }

    private function qglValidations($row, $request, $parasupri, &$result)
    {
        try {
            if (!count($result)) {
                $row['IDMARSEMQUA'] = $parasupri['IDMARSEMQUA'];
                $row['IDULTIDTVALQ'] = $parasupri['IDULTIDTVALQ'];

                $qfomarprod = $this->getQfomarprod($row, $request);
                if (!$this->isThereRegisteredData($qfomarprod)) {
                    $result = array(
                        "ROUTE"   => "gql_cadastros#gql01200_qualificacao_de_marcas_por_produto",
                        "ERROR"   => "QFOMARPROD_ERROR",
                        "WINDOW"  => "QFOMARPROD"
                    );
                }

                if (!count($result)) {
                    $qfoforn = $this->getQfoforn($row, $request);
                    if (!$this->isThereRegisteredData($qfoforn)) {
                        $result = array(
                            "ROUTE"   => "gql_cadastros#gql00703_qualificacao_de_fornecedores",
                            "ERROR"   => "QFOFORN_ERROR",
                            "WINDOW"  => "QFOFORN"
                        );
                    }
                }

                if (!count($result)) {
                    $qfoprodforn = $this->getQfoprodforn($row, $request);
                    if ($this->isThereRegisteredData($qfoprodforn)) {
                        $result = array(
                            "ROUTE"   => "gql_cadastros#gql01800_qualificacao_de_produto_por_fornecedor",
                            "ERROR"   => "QFOPRODFORN_ERROR",
                            "WINDOW"  => "QFOPRODFORN"
                        );
                    }
                }
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    public function verificarPlanilhaCotacao(DTO\Request $request, DTO\Response $response)
    {
        $row = $request->getRow();
        $result = array();

        try {
            $unitData = $this->unitValidations($row, $request, $result);
            $this->productValidations($row, $request, $result);
            $parasupri = $this->parasupriValidations($row, $request, end($unitData), $result);
            $this->fornecedorValidations($row, $request, $result);
            $this->prodFornValidations($row, $request, $result);
            $this->marcprodValidations($row, $request, $result);
            $this->prodcompValidations($row, $request, $result);
            $this->qglValidations($row, $request, end($parasupri), $result);

            if (!count($result)) {
                $result = array("ERROR" => "");
            }

            $response->addDataSet(new DataSet('data', $result));
        } catch (Exception $e) {
            $response->setError($e->getMessage());
        }
    }

    private function getParcmpsup($row, $request)
    {
        try {
            $this->dataSourceName = 'parcmpsup';
            $filterCriteria = new FilterCriteria($this->dataSourceName);
            $filterCriteria->addCondition('CDFILIAL', $row['CDFILIAL']);
            return $this->getData($filterCriteria, $request);
        } catch (Exception $e) {
            throw $e;
        }
    }

    private function parcmpsupValidations($row, $request, &$result)
    {
        try {
            if (!count($result)) {
                $parcmpsup = $this->getParcmpsup($row, $request);
                if (!$this->isThereRegisteredData($parcmpsup)) {
                    $result = array(
                        "ROUTE"   => "sup_parametros#cmp00600_parametros_de_compras_e_suprimentos",
                        "ERROR"   => "PARCMPSUP_ERROR",
                        "WINDOW"  => "PARASUPRI"
                    );
                }

                return $parcmpsup;
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    private function getProdprefmed($row, $request)
    {
        try {
            $this->dataSourceName = 'prodprefmed';
            $filterCriteria = new FilterCriteria($this->dataSourceName);
            $filterCriteria->addCondition('CDFILIAL', $row['CDFILIAL']);
            $filterCriteria->addCondition('CDPRODUTO', $row['CDPRODESTO']);
            return $this->getData($filterCriteria, $request);
        } catch (Exception $e) {
            throw $e;
        }
    }

    private function prodprefmedValidations($row, $request, $parcmpsup, &$result)
    {
        try {
            if (!count($result) && $parcmpsup['IDUTPRODPREF'] === 'S') {
                $prodprefmed = $this->getProdprefmed($row, $request);
                if ($this->isThereRegisteredData($prodprefmed)) {
                    if (!end($prodprefmed)['QTMEDSOLPROD']) {
                        $result = array(
                            "ROUTE"   => "sup_cadastros#cmp02900_produtopreferencial",
                            "ERROR"   => "PRODPREFMED_ERROR",
                            "WINDOW"  => "PRODPREFMED"
                        );
                    }
                }
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    private function getColetapr($row, $request)
    {
        try {
            $this->dataSourceName = 'exists_itemcole';
            $filterCriteria = new FilterCriteria($this->dataSourceName);
            $filterCriteria->addCondition('CDFILIAL', $row['CDFILCENTCMP']);
            $filterCriteria->addCondition('CDFORNECED', $row['CDFORNECED']);
            $filterCriteria->addCondition('CDREGIAO', $row['CDREGIAO']);
            $filterCriteria->addCondition('CDPRODUTO', $row['CDPRODUTO']);
            $filterCriteria->addCondition('CDMARCA', $row['CDMARCA']);
            $filterCriteria->addCondition('DTINIVALITM', $row['INTERVAL_DT']['start']);
            $filterCriteria->addCondition('DTFINVALITM', $row['INTERVAL_DT']['end']);
            return $this->getData($filterCriteria, $request);
        } catch (Exception $e) {
            throw $e;
        }
    }

    private function coletaprValidations($row, $request, $parasupri, &$result)
    {
        try {
            if (!count($result)) {
                $itemcole = $this->getColetapr($row, $request);
                if (!$this->isThereRegisteredData($itemcole)) {
                    $result = array(
                        "ROUTE"   => "sup_cotacao_planilhaCotacao#sup05200_planilha_de_cotacao",
                        "ERROR"   => "COLETAPR_ERROR",
                        "WINDOW"  => "COLETAPR"
                    );
                }

                if (!count($result) && $parasupri['IDUTISTATCOT'] === 'S' && is_array($itemcole) && count($itemcole)) {
                    $data = array_filter($itemcole, function ($row) {
                        return $row['IDSITUCOLETA'] === 'A';
                    }, ARRAY_FILTER_USE_BOTH);

                    $isThereData = is_array($data) && count($data);

                    if (!$isThereData) {
                        $result = array(
                            "ROUTE"   => "sup_cotacao_planilhaCotacao#sup05200_planilha_de_cotacao",
                            "ERROR"   => "IDSITUCOLETA_ERROR",
                            "WINDOW"  => "COLETAPR"
                        );
                    }
                }
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    public function verificarApuracaoEfetivacao(DTO\Request $request, DTO\Response $response)
    {
        $row = $request->getRow();
        $result = array();

        try {
            $unitData = $this->unitValidations($row, $request, $result);
            $parasupri = $this->parasupriValidations($row, $request, end($unitData), $result);
            $this->productValidations($row, $request, $result);
            $this->fornecedorValidations($row, $request, $result);
            $this->prodFornValidations($row, $request, $result);
            $this->marcprodValidations($row, $request, $result);
            $this->prodcompValidations($row, $request, $result);
            $this->qglValidations($row, $request, end($parasupri), $result);
            $parcmpsup = $this->parcmpsupValidations($row, $request, $result);
            $this->prodprefmedValidations($row, $request, end($parcmpsup), $result);
            $this->coletaprValidations($row, $request, end($parasupri), $result);

            if (!count($result)) {
                $result = array("ERROR" => "");
            }

            $response->addDataSet(new DataSet('data', $result));
        } catch (Exception $e) {
            $response->setError($e->getMessage());
        }
    }

    public function verificarLancamentoAf(DTO\Request $request, DTO\Response $response)
    {
        $row = $request->getRow();
        $result = array();

        try {

            $unitData = $this->unitValidations($row, $request, $result);
            $parasupri = $this->parasupriValidations($row, $request, end($unitData), $result);
            $this->fornecedorValidations($row, $request, $result);
            $this->prodFornValidations($row, $request, $result);
            $this->marcprodValidations($row, $request, $result);
            $this->qglValidations($row, $request, end($parasupri), $result);
            $this->prodcompValidations($row, $request, $result);
            $parcmpsup = $this->parcmpsupValidations($row, $request, $result);
            $this->prodprefmedValidations($row, $request, end($parcmpsup), $result);

            if (!count($result)) {
                $result = array("ERROR" => "");
            }

            $response->addDataSet(new DataSet('data', $result));
        } catch (Exception $e) {
            $response->setError($e->getMessage());
        }
    }

    private function getPartfili($row, $request)
    {
        try {
            $this->dataSourceName = 'partfili';
            $filterCriteria = new FilterCriteria($this->dataSourceName);
            $filterCriteria->addCondition('CDFILIAL', $row['CDFILIAL']);
            $filterCriteria->addCondition('CDPRODUTO', $row['CDPRODUTO']);
            return $this->getData($filterCriteria, $request);
        } catch (Exception $e) {
            throw $e;
        }
    }

    private function partfiliValidations($params, $request, &$result)
    {
        try {
            if (!count($result)) {
                $partfili = $this->getPartfili($params, $request);
                if ($this->isThereRegisteredData($partfili)) {
                    if (in_array(end($partfili)['CDTIPPART'], array("1", "2", "3"))) {
                        $data = array_filter($partfili, function ($row) use ($params) {
                            return ($row['CDTIPPART'] === '3' && $row['CDMARCA'] === $params['CDMARCA']);
                        }, ARRAY_FILTER_USE_BOTH);

                        if ($this->isThereRegisteredData($data)) {
                            $result = array(
                                "ROUTE"   => "sup_cadastros#cmp02600_particularidades",
                                "ERROR"   => "PARTFILI_RM_ERROR",
                                "WINDOW"  => "PARTFILI"
                            );
                        }

                        if (!count($result)) {
                            $data = array_filter($partfili, function ($row) use ($params) {
                                return ($row['CDTIPPART'] === '1' && $row['CDMARCA'] !== $params['CDMARCA']);
                            }, ARRAY_FILTER_USE_BOTH);

                            if ($this->isThereRegisteredData($data)) {
                                $result = array(
                                    "ROUTE"   => "sup_cadastros#cmp02600_particularidades",
                                    "ERROR"   => "PARTFILI_MO_ERROR",
                                    "WINDOW"  => "PARTFILI"
                                );
                            }
                        }

                        if (!count($result)) {
                            $data = array_filter($partfili, function ($row) use ($params) {
                                return ($row['CDTIPPART'] === '2' && $row['CDMARCA'] !== $params['CDMARCA']);
                            }, ARRAY_FILTER_USE_BOTH);

                            if ($this->isThereRegisteredData($data)) {
                                $result = array(
                                    "ROUTE"   => "sup_cadastros#cmp02600_particularidades",
                                    "ERROR"   => "PARTFILI_MP_ERROR",
                                    "WINDOW"  => "PARTFILI"
                                );
                            }
                        }
                    }

                    if (in_array(end($partfili)['CDTIPPART'], array("4", "5", "6"))) {
                        $data = array_filter($partfili, function ($row) use ($params) {
                            return ($row['CDTIPPART'] === '6' && $row['CDFORNECED'] === $params['CDFORNECED']);
                        }, ARRAY_FILTER_USE_BOTH);

                        if ($this->isThereRegisteredData($data)) {
                            $result = array(
                                "ROUTE"   => "sup_cadastros#cmp02600_particularidades",
                                "ERROR"   => "PARTFILI_RF_ERROR",
                                "WINDOW"  => "PARTFILI"
                            );
                        }

                        if (!count($result)) {
                            $data = array_filter($partfili, function ($row) use ($params) {
                                return ($row['CDTIPPART'] === '4' && $row['CDFORNECED'] !== $params['CDFORNECED']);
                            }, ARRAY_FILTER_USE_BOTH);

                            if ($this->isThereRegisteredData($data)) {
                                $result = array(
                                    "ROUTE"   => "sup_cadastros#cmp02600_particularidades",
                                    "ERROR"   => "PARTFILI_FO_ERROR",
                                    "WINDOW"  => "PARTFILI"
                                );
                            }
                        }

                        if (!count($result)) {
                            $data = array_filter($partfili, function ($row) use ($params) {
                                return ($row['CDTIPPART'] === '5' && $row['CDFORNECED'] !== $params['CDFORNECED']);
                            }, ARRAY_FILTER_USE_BOTH);

                            if ($this->isThereRegisteredData($data)) {
                                $result = array(
                                    "ROUTE"   => "sup_cadastros#cmp02600_particularidades",
                                    "ERROR"   => "PARTFILI_FP_ERROR",
                                    "WINDOW"  => "PARTFILI"
                                );
                            }
                        }
                    }

                    if (in_array(end($partfili)['CDTIPPART'], array("7", "8", "9"))) {
                        $data = array_filter($partfili, function ($row) use ($params) {
                            return ($row['CDTIPPART'] === '9' && ($row['CDMARCA'] === $params['CDMARCA'] || $row['CDFORNECED'] === $params['CDFORNECED']));
                        }, ARRAY_FILTER_USE_BOTH);

                        if ($this->isThereRegisteredData($data)) {
                            $result = array(
                                "ROUTE"   => "sup_cadastros#cmp02600_particularidades",
                                "ERROR"   => "PARTFILI_RMF_ERROR",
                                "WINDOW"  => "PARTFILI"
                            );
                        }

                        if (!count($result)) {
                            $data = array_filter($partfili, function ($row) use ($params) {
                                return ($row['CDTIPPART'] === '7' && ($row['CDMARCA'] !== $params['CDMARCA'] || $row['CDFORNECED'] !== $params['CDFORNECED']));
                            }, ARRAY_FILTER_USE_BOTH);

                            if ($this->isThereRegisteredData($data)) {
                                $result = array(
                                    "ROUTE"   => "sup_cadastros#cmp02600_particularidades",
                                    "ERROR"   => "PARTFILI_MFO_ERROR",
                                    "WINDOW"  => "PARTFILI"
                                );
                            }
                        }

                        if (!count($result)) {
                            $data = array_filter($partfili, function ($row) use ($params) {
                                return ($row['CDTIPPART'] === '8' && ($row['CDMARCA'] !== $params['CDMARCA'] || $row['CDFORNECED'] !== $params['CDFORNECED']));
                            }, ARRAY_FILTER_USE_BOTH);

                            if ($this->isThereRegisteredData($data)) {
                                $result = array(
                                    "ROUTE"   => "sup_cadastros#cmp02600_particularidades",
                                    "ERROR"   => "PARTFILI_MFP_ERROR",
                                    "WINDOW"  => "PARTFILI"
                                );
                            }
                        }
                    }
                }
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    public function verificarCustoLista(DTO\Request $request, DTO\Response $response)
    {
        $row = $request->getRow();
        $result = array();

        try {
            $unitData = $this->unitValidations($row, $request, $result);
            $parasupri = $this->parasupriValidations($row, $request, end($unitData), $result);
            $parcmpsup = $this->parcmpsupValidations($row, $request, $result);
            $this->fornecedorValidations($row, $request, $result);
            $this->qglValidations($row, $request, end($parasupri), $result);
            $this->prodprefmedValidations($row, $request, end($parcmpsup), $result);

            $this->productValidations($row, $request, $result);
            $this->prodFornValidations($row, $request, $result);
            $this->marcprodValidations($row, $request, $result);
            $this->prodcompValidations($row, $request, $result);

            $this->coletaprValidations($row, $request, end($parasupri), $result);
            $this->partfiliValidations($row, $request, $result);

            if (!count($result)) {
                $result = array("ERROR" => "");
            }

            $response->addDataSet(new DataSet('data', $result));
        } catch (Exception $e) {
            $response->setError($e->getMessage());
        }
    }

    private function getReportData($filter)
    {
        try {
            $filter->setDataSourceName($this->dataSourceName);
            $newFilterCriteria = $this->crudService->getPopulatedFilterCriteria($filter, array());
            $dataSource = $this->dataSourceManager->findBy($newFilterCriteria);
            return $dataSource->getRows();
        } catch (Exception $e) {
            throw $e;
        }
    }

    public function GridRelatorio(Request\Filter $request, Response $response)
    {
        $filter = $request->getFilterCriteria();
        
        try {
            $dataSet = $this->getReportData($filter);        

            if (empty($dataSet)) {
                $msg = 'Produto informado não está na lista ou não possui cotação.';
                $response->addDataSet(new DataSet('sup08200_relatorio', $dataSet));
            } else {
                $status = explode('-', $dataSet[0]['IDSTATUS']);

                if ($status[0] == 1) {
                    $status[0] = 'OK';
                    //$response->addMessage(new Message('Operação realizada com sucesso!'));
                } else {
                    $status[0] = 'Produto Preferencial';
                }

                if ($status[1] == 1) {
                    $status[1] = '';
                } else {
                    $status[1] = '_Fornecedor não Atende a Unidade';
                }

                if ($status[2] == 1) {
                    $status[2] = '';
                } else {
                    $status[2] = '_Fornecedor não Atende a Região';
                }

                if ($status[3] == 1) {
                    $status[3] = '';
                } else {
                    $status[3] = '_Qualificação de Produto por Fornecedor';
                }

                if ($status[4] == 1) {
                    $status[4] = '';
                } else {
                    $status[4] = '_Qualificação da Marca';
                }

                if ($status[5] == 1) {
                    $status[5] = '';
                } else {
                    $status[5] = '_Qualificação da Marca';
                }

                if ($status[6] == 1) {
                    $status[6] = '';
                } else {
                    $status[6] = '_Cadastro de Fornecedor (alternativo, inativo e/ou desqualificado)';
                }

                if ($status[7] == 1) {
                    $status[7] = '';
                } else {
                    $status[7] = '_Restrição à Marca';
                }

                if ($status[8] == 1) {
                    $status[8] = '';
                } else {
                    $status[8] = '_Restrição ao Fornecedor';
                }

                if ($status[9] == 1) {
                    $status[9] = '';
                } else {
                    $status[9] = '_Restrição à marca/fornecedor';
                }

                if ($status[10] == 1) {
                    $status[10] = '';
                } else {
                    $status[10] = '_Particularidade do Produto';
                }

                for ($i = 0, $j = count($dataSet); $i < $j; $i++) {
                    $dataSet[$i]['IDSTATUS'] = str_replace('-', '', (join('-', $status)));
                }

                $response->addDataSet(new DataSet('sup08200_relatorio', $dataSet));
            }
        } catch (Exception $e) {
            $response->setError($e->getMessage());
        }
    }
}
